;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

;; Place your private configuration here! Remember, you do not need to run 'doom
;; sync' after modifying this file!


;; Some functionality uses this to identify you, e.g. GPG configuration, email
;; clients, file templates and snippets. It is optional.
;; (setq user-full-name "John Doe"
;;       user-mail-address "john@doe.com")

;; Doom exposes five (optional) variables for controlling fonts in Doom:
;;
;; - `doom-font' -- the primary font to use
;; - `doom-variable-pitch-ont' -- a non-monospace font (where applicable)
;; - `doom-big-font' -- used for `doom-big-font-mode'; use this for
;;   presentations or streaming.
;; - `doom-unicode-font' -- for unicode glyphs
;; - `doom-serif-font' -- for the `fixed-pitch-serif' face
;;
;; See 'C-h v doom-font' for documentation and more examples of what they
;; accept. For example:
;;

(setq nerd-icons-font-names '("SymbolsNerdFontMono-Regular.ttf"))

(setq doom-font (font-spec :family "Iosevka Nerd Font Mono" :size 14)
      doom-variable-pitch-font (font-spec :family "Iosevka Nerd Font Propo" :size 14)
      ;; doom-symbol-font (font-spec :family "all-the-icons" :size 13)
      doom-big-font (font-spec :family "Iosevka Nerd Font Mono" :size 18 :weight 'demibold))

(defun add-back-emoji-fallback-font-families ()
  (when (fboundp 'set-fontset-font)
    (let ((fn (doom-rpartial #'member (font-family-list))))
      (when-let (font (cl-find-if fn doom-emoji-fallback-font-families))
        (set-fontset-font t 'unicode font nil 'append)))))

;; (add-hook 'after-setting-font-hook 'add-back-emoji-fallback-font-families)

;;
;; If you or Emacs can't find your font, use 'M-x describe-font' to look them
;; up, `M-x eval-region' to execute elisp code, and 'M-x doom/reload-font' to
;; refresh your font settings. If Emacs still can't find your font, it likely
;; wasn't installed correctly. Font issues are rarely Doom issues!

;; There are two ways to load a theme. Both assume the theme is installed and
;; available. You can either set `doom-theme' or manually load a theme with the
;; `load-theme' function. This is the default:
(setq doom-theme 'doom-gruvbox)

;; This determines the style of line numbers in effect. If set to `nil', line
;; numbers are disabled. For relative line numbers, set this to `relative'.
(setq display-line-numbers-type t)

;; If you use `org' and don't want your org files in the default location below,
;; change `org-directory'. It must be set before org loads!
(setq org-directory "~/org/")


;; Whenever you reconfigure a package, make sure to wrap your config in an
;; `after!' block, otherwise Doom's defaults may override your settings. E.g.
;;
;;   (after! PACKAGE
;;     (setq x y))
;;
;; The exceptions to this rule:
;;
;;   - Setting file/directory variables (like `org-directory')
;;   - Setting variables which explicitly tell you to set them before their
;;     package is loaded (see 'C-h v VARIABLE' to look up their documentation).
;;   - Setting doom variables (which start with 'doom-' or '+').
;;
;; Here are some additional functions/macros that will help you configure Doom.
;;
;; - `load!' for loading external *.el files relative to this one
;; - `use-package!' for configuring packages
;; - `after!' for running code after a package has loaded
;; - `add-load-path!' for adding directories to the `load-path', relative to
;;   this file. Emacs searches the `load-path' when you load packages with
;;   `require' or `use-package'.
;; - `map!' for binding new keys
;;
;; To get information about any of these functions/macros, move the cursor over
;; the highlighted symbol at press 'K' (non-evil users must press 'C-c c k').
;; This will open documentation for it, including demos of how they are used.
;; Alternatively, use `C-h o' to look up a symbol (functions, variables, faces,
;; etc).
;;
;; You can also try 'gd' (or 'C-c c d') to jump to their definition and see how
;; they are implemented.
;; TRAMP


;; (add-to-list 'tramp-connection-properties
;;              (list (regexp-quote "/sshx:user@host:")
;;                    "remote-shell" "/usr/bin/fish"))
;;
;; (eval-after-load 'tramp
;;  '((setenv "SHELL" "/run/current-system/sw/bin/fish")
;;    (setenv "ESHELL" "/run/current-system/sw/bin/fish")
;;    (add-to-list 'tramp-remote-path
;;                 "/run/current-system/sw/bin")
;;    )
;;  )
;; (setq! tramp-default-method "ssh")

;; rust
(setq! lsp-inlay-hint-enable t)
(setq! lsp-rust-analyzer-cargo-run-build-scripts t)
(setq! lsp-rust-analyzer-proc-macro-enable t)
(setq! lsp-rust-analyzer-server-display-inlay-hints t)
(setq! lsp-rust-analyzer-completion-auto-import-enable t)

;; llama.cpp


;; openchat prompt: GPT4 User: {prompt}<|end_of_turn|>GPT4 Assistant:

(use-package llama-cpp
  :ensure t)

;; (use-package llama-cpp-code)
;; (use-package llama-cpp-chat)


;; openchat
;; (setq!
;;  llama-cpp-params '(
;;                     :n_predict -1
;;                     :stop ["<|end_of_turn|>"]
;;                     )

;;  llama-cpp-chat-prompt "System:
;; You are an intelligent programming assistant.
;; The User is a professional software developer, be concise in your answers and assume technical proficiency.
;; "
;;  llama-cpp-chat-input-prefix "
;; GPT4User:
;; "
;;  llama-cpp-chat-input-suffix "
;; <|end_of_turn|>
;; GPT4 Assistant:
;; ")

;; Phind codellama :
(setq! llama-cpp-chat-prompt "### System Prompt
You are an intelligent programming assistant."
       llama-cpp-chat-input-prefix "

### User Message
"
       llama-cpp-chat-input-suffix "

### Assistant
")


;; Nix

(use-package! nixpkgs-fmt
  :hook (nix-mode . nixpkgs-fmt-on-save-mode))

;; Shell stuff;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq shell-file-name (executable-find "bash"))

(setq-default vterm-shell (executable-find "fish"))

;; Background Transparency + Toggling

(defun set-frame-bg-opacity (opacity)
                                        ;Interactively change the current frames background opacity.
                                        ;     OPACITY is an integer between 0 to 100, inclusive.
  (interactive
   (list (read-number "Opacity (0-100): "
                      (or (frame-parameter nil 'alpha)
                          100))))
  (set-frame-parameter nil 'alpha-background opacity))


(defun toggle-bg-transparency ()
                                        ;Toggle between transparent and opaque background.
  (interactive)
  (let ((alpha-background (frame-parameter nil 'alpha-background)))
    (set-frame-bg-opacity
     (if (eql (cond
               ((numberp alpha-background) alpha-background)
               (t 100))
              100)
         85 100)
     )))

(map! :leader
      (:prefix-map ("t" . "toggle")
                   (:prefix ("T" . "Transparancy")
                    :desc "Toggle background transparancy" "b" #'toggle-bg-transparency)))

(defun setup-frame-transparency (frame)
  (set-frame-parameter frame 'alpha-background 85))

(add-hook 'after-make-frame-functions #'setup-frame-transparency)


;; TABS;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq! uniquify-separator "/")
(setq! uniquify-buffer-name-style 'forward)

(use-package centaur-tabs
  :init
  (setq centaur-tabs-enable-key-bindings t)
  :config
  (setq centaur-tabs-style "wave"
        centaur-tabs-height 32
        centaur-tabs-set-icons t
        centaur-tabs-show-new-tab-button t
        centaur-tabs-show-navigation-buttons t
        centaur-tabs-set-bar 'under
        centaur-tabs-show-count t
        centaur-tabs-gray-out-icons 'buffer
        x-underline-at-descent-line t
        centaur-tabs-set-modified-marker t
        centaur-tabs-modified-marker "•"
        centaur-tabs-close-button "x"
        centaur-tabs-cycle-scope 'tabs
        centaur-tabs-adjust-buffer-order t
        centaur-tabs-left-edge-margin nil)

  (centaur-tabs-change-fonts (face-attribute 'default :font) 110)
  (centaur-tabs-headline-match)
  (centaur-tabs-group-by-projectile-project)
  ;; (centaur-tabs-enable-buffer-alphabetical-reordering)
  (centaur-tabs-mode t)


  (defvar categories-alist
    '(
      (:name "Terminal" :modes (vterm-mode term-mode) :type per-project)
      (:name "Org" :modes (org-mode org-agenda-mode) :type global)
      (:name "Misc" :modes (+doom-dashboard-mode native-comp-limple-mode messages-buffer-mode) :type global)
      (:name "Misc" :modes (special-mode fundamental-mode) :type per-project)
      )
    "Mapping of categories to modes with their respective type logic.")

  (defun get-project-name-or-default (default-name)
    "Return the project name if available, or DEFAULT-NAME."
    (if-let ((project-root (condition-case _err
                               (projectile-project-root)
                             (error nil))))
        (concat (projectile-project-name) "::" default-name)
      default-name))

  (defun get-category-name (mode)
    "Get category name for the given MODE."
    (let (category-name)
      (dolist (category categories-alist category-name)
        (let ((modes (plist-get category :modes)))
          (when (or (null modes) (memq mode modes))
            (setq category-name
                  (if (eq (plist-get category :type) 'per-project)
                      (get-project-name-or-default (plist-get category :name))
                    (plist-get category :name))))))))

  (defun centaur-tabs-group-name-by-category ()
    "Get tab group name based on category."
    (let ((mode-category (get-category-name major-mode)))
      (if mode-category
          (list mode-category)
        (list (get-project-name-or-default "Main")))))

  (defun centaur-tabs-projectile-buffer-groups ()
    "Return the list of group names BUFFER belongs to."
    (if centaur-tabs-projectile-buffer-group-calc
        (symbol-value 'centaur-tabs-projectile-buffer-group-calc)
      (set (make-local-variable 'centaur-tabs-projectile-buffer-group-calc)
           (centaur-tabs-group-name-by-category))))

  (add-hook! '(+doom-dashboard-mode-hook +popup-buffer-mode-hook)
    (defun +tabs-disable-centaur-tabs-mode-maybe-h ()
      "Disable `centaur-tabs-mode' in current buffer."
      (when (centaur-tabs-mode-on-p)
        (centaur-tabs-local-mode))))
  )

;; cape ;;;;;

(use-package! cape)

;; (after! company
;;  (setq-default company-frontends '(company-preview-frontend)))

(use-package! codeium
  :after cape
  :init
  ;; use globally
  (add-to-list 'completion-at-point-functions #'codeium-completion-at-point)
  ;; or on a hook
  ;; (add-hook 'python-mode-hook
  ;;     (lambda ()
  ;;         (setq-local completion-at-point-functions '(codeium-completion-at-point))))

  ;; if you want multiple completion backends, use cape (https://github.com/minad/cape):
  ;; (add-hook 'python-mode-hook
  ;;     (lambda ()
  ;;         (setq-local completion-at-point-functions
  ;;             (list (cape-super-capf #'codeium-completion-at-point #'lsp-completion-at-point)))))
  ;; an async company-backend is coming soon!

  ;; codeium-completion-at-point is autoloaded, but you can
  ;; optionally set a timer, which might speed up things as the
  ;; codeium local language server takes ~0.2s to start up
  ;; (add-hook 'emacs-startup-hook
  ;;  (lambda () (run-with-timer 0.1 nil #'codeium-init)))

  ;; :defer t ;; lazy loading, if you want
  :config
  (setq use-dialog-box nil) ;; do not use popup boxes

  ;; if you don't want to use customize to save the api-key
  ;; (setq codeium/metadata/api_key "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx")
  (setq codeium/metadata/api_key (with-temp-buffer (insert-file-contents "~/.codeium")
                                                   (buffer-string)))

  (defalias 'my/codeium-complete
    (cape-interactive-capf #'codeium-completion-at-point))

  (map! :localleader
        :map evil-normal-state-map
        "c e" #'my/codeium-complete)

  ;; get codeium status in the modeline
  (setq codeium-mode-line-enable
        (lambda (api) (not (memq api '(CancelRequest Heartbeat AcceptCompletion)))))
  (add-to-list 'mode-line-format '(:eval (car-safe codeium-mode-line)) t)
  ;; alternatively for a more extensive mode-line
  ;; (add-to-list 'mode-line-format '(-50 "" codeium-mode-line) t)

  ;; use M-x codeium-diagnose to see apis/fields that would be sent to the local language server
  (setq codeium-api-enabled
        (lambda (api)
          (memq api '(GetCompletions Heartbeat CancelRequest GetAuthToken RegisterUser auth-redirect AcceptCompletion))))
  ;; you can also set a config for a single buffer like this:
  ;; (add-hook 'python-mode-hook
  ;;     (lambda ()
  ;;         (setq-local codeium/editor_options/tab_size 4)))

  ;; You can overwrite all the codeium configs!
  ;; for example, we recommend limiting the string sent to codeium for better performance
  (defun my-codeium/document/text ()
    (buffer-substring-no-properties (max (- (point) 3000) (point-min)) (min (+ (point) 1000) (point-max))))
  ;; if you change the text, you should also change the cursor_offset
  ;; warning: this is measured by UTF-8 encoded bytes
  (defun my-codeium/document/cursor_offset ()
    (codeium-utf8-byte-length
     (buffer-substring-no-properties (max (- (point) 3000) (point-min)) (point))))
  (setq codeium/document/text 'my-codeium/document/text)
  (setq codeium/document/cursor_offset 'my-codeium/document/cursor_offset))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
