{ config
, lib
, pkgs
, ...
}: {
  programs.dconf.enable = true;

  # fonts

  fonts.fontDir.enable = true;
  fonts.packages = with pkgs; [
    emacs-all-the-icons-fonts
    corefonts
    vistafonts
    noto-fonts
    noto-fonts-cjk
    noto-fonts-emoji
    liberation_ttf
    fira
    fira-code
    fira-code-symbols
    mplus-outline-fonts.githubRelease
    dina-font
    proggyfonts
    (nerdfonts.override { fonts = [ "Iosevka" "IosevkaTerm" "FiraCode" "DroidSansMono" "NerdFontsSymbolsOnly" ]; })
  ];

  networking.firewall = {
    enable = true;
    allowedTCPPortRanges = [
      {
        from = 1714;
        to = 1764;
      } # KDE Connect
    ];
    allowedUDPPortRanges = [
      {
        from = 1714;
        to = 1764;
      } # KDE Connect
    ];
  };

  services = {
    xserver = {
      enable = true;
      layout = "us";
      xkbVariant = "alt-intl";

      desktopManager.plasma5.enable = true;

      displayManager = {
        defaultSession = "plasmawayland";

        sddm.enable = true;

        # autoLogin = {
        #  enable = true;
        #  user = "tristand";
        # };
      };
    };
    pipewire = {
      enable = true;
      alsa.enable = true;
      alsa.support32Bit = true;
      pulse.enable = true;
      jack.enable = true;
    };
    #########

    ## printing

    printing = {
      enable = true;
      drivers = with pkgs; [
        gutenprint
        gutenprintBin
        hplipWithPlugin
        brlaser
        brgenml1cupswrapper
      ];
    };
  };

  ## audio
  sound.enable = true;
  hardware.bluetooth.enable = true;

  environment.systemPackages = with pkgs; [
    cifs-utils
  ];

  programs.steam = {
    enable = true;
    remotePlay.openFirewall = true;
    dedicatedServer.openFirewall = true;
  };

  chaotic.steam.extraCompatPackages = with pkgs; [
    proton-ge-custom
  ];
}
